/****************************************************************************
*                                                                           *
* Program: DUPDISK.C                                                        *
*                                                                           *
* Purpose: Allows duplication of a diskette by reading the source diskette  *
*          only once to a temporary file and then copying the image to a    *
*          target diskette of the same type.                                *
*                                                                           *
*  Author: Frank Ray Kinser (CIS 73767,3071)                                *
*                                                                           *
* Version: 1.0 (8-23-92)                                                    *
*                                                                           *
* Dialect: Borland C++ 3.0 with FUNCky II Development Library.              *
*                                                                           *
*   Notes: 1. Does not work well with device driven drives (e.g. Ontrack's  *
*             Diskmanager, etc.                                             *
*                                                                           *
****************************************************************************/

#include "FUNCky.h"  


/***********************
*                      *
*  External Functions  *
*                      *
***********************/

extern int ChkTgtDrv( int TgtDrvNum, int TgtDrvType ) ;


/*********************
*                    *
*  Global Variables  *
*                    *
*********************/

int  SrcDrvNum    = 0 ;   // Drive letter converted to physical drive number.
int  SrcDrvType   = 0 ;   // Media type of the source drive: 360, 720 144, etc.

int  TmpDrvNum    = 0 ;   // Temporary drive number.
char TmpFile[18] ;        // Filename to store temporary image.


/*****************
*                *
*  Main Program  *
*                *
*****************/

void main( int argc, char *argv[] )
{

   if( argc < 3 )   // Command line syntax checking.
   {
      _stdoutline( "Usage: DUPDISK <sourcedrive> <tempdrive> (e.g. DUPDISK A: C:) ") ;
      exit(0) ;
   }

   if( (_ati(argv[1],"A:B:")) == ERROR )
   {
      _stdoutline( "Source drive must be A: or B: " ) ;
      exit(0) ;
   }


   if( !_isfixed(argv[2]) )   // Check temporary holding drive.
   {
      _stdoutline( "Temporary holding drive must be a hard disk " ) ;
      exit(0) ;
   }


   /**********************************
   *                                 *
   *  Copy contents of source disk.  *
   *                                 *
   **********************************/

   _stdout("Insert Source disk in drive and strike any key..." ) ;
   _inkey(0) ;
   _stdoutline("") ;

   while( _isdrive(argv[1]) == FALSE )  // Make sure the drive is ready.
   {
      _stdout( "Please check drive or press ESC to abort..." ) ;

      if( (_inkey(0)) == 0x1b )  // User bailed out.
         exit(0) ;
      else
         _stdoutline("") ;

   }

   SrcDrvNum   = _dosdrive( argv[1] ) ;  // Get the physical number of the source drive.

   SrcDrvType  = _disktype( SrcDrvNum ) ;  // Get the drive type.

   _strcpy( TmpFile, argv[2] ) ;         // Create temporary file name.
   _strcat( TmpFile, "\diskette.img" ) ;

   _stdout( "Copying diskette image to temporary file..." ) ;
   if( ( _disk2file( SrcDrvNum, TmpFile ) ) == FALSE )
   {
      _stdoutline( "Error in copying diskette to temporary holding disk" ) ;
      exit(0) ;
   }
   else
      _stdoutline("") ;


   /*****************************************
   *                                        *
   *  Copy temporary file back to diskette  *
   *                                        *
   *****************************************/

   while(1)
   {

      if( (ChkTgtDrv( SrcDrvNum, SrcDrvType )) == 0 )
      {
         _stdout( "Copying diskette image to target diskette..." ) ;

         if( (_file2disk( SrcDrvNum, TmpFile )) == FALSE )
         {
            _stdoutline( "Unknown error in copying image to diskette" ) ;
            _ferase( TmpFile ) ;
            exit(0) ;
         }
         else
         {
            _stdoutline("") ;
            _stdout("Dupdisk Complete. Press ESC to quit or any other key to copy another disk...") ;
            if( (_inkey(0)) == 0x1b )
            {
               _ferase( TmpFile ) ;
               exit(0) ;
            }
            else
               _stdoutline("") ;
         }
      }

   }

}


/*******************************************************************************
*                                                                              *
* Function: int ChkTgtDrv( int TgtDrvNum, int TgtDrvType )                     *
*                                                                              *
*  Purpose: To check the target diskette to make sure it is ready for writing. *
*           If the diskette is unformatted ChkTgtDrv will format the diskette, *
*           as well as other common errors.                                    *
*                                                                              *
*    Parms: TgtDrvNum  - The number of the physical drive (e.g. 0=A, 1=B,      *
*                        etc.)                                                 *
*                                                                              *
*           TgtDrvType - The type of drive as defined by the Funcky function   *
*                        _disktype().                                          *
*                                                                              *
*  Returns: Zero if diskette is ready or non-zero if not.                      *
*                                                                              *
*******************************************************************************/

int ChkTgtDrv( int TgtDrvNum, int TgtDrvType )
{

   int retval = 0 ;  // Returning value from function. Assume 0 (OK).

   _stdout("Insert blank diskette of same media type in drive and press any key..." ) ;
   _inkey(0) ; _stdoutline("") ;


   while( (_diskstatus( TgtDrvNum )) != 0 )  // Diskette is not ready.
   {

      switch( _diskstatus( TgtDrvNum ) )
      {
         case (1) :

            _stdout( "Drive is write protected. Correct and retry" ) ;
            _inkey(0) ; _stdoutline("") ; break ;

         case (3) :

            _stdout( "Drive door may be open. Correct and retry" ) ;
            _inkey(0) ; _stdoutline("") ; break ;

         case (13) :

            _stdout( "Formatting diskette..." ) ;
            if( (_diskformat( TgtDrvNum, TgtDrvType, NULL )) != 0 )
            {
               _stdoutline("") ;
               _stdoutline( "Error: Diskette format failure" ) ;
               retval = 13 ;
            }
            else
               _stdoutline("") ;

            break ;

         default :

            _stdoutline("") ;
            _stdoutline("Unknown Diskette Error") ;
            break ;
      }

   }

   if( TgtDrvType != _disktype( TgtDrvNum ) )   // Recursive call to ChkRgtDrv()
   {
      _stdoutline("Target diskette does not match source...") ;
      retval = ChkTgtDrv( TgtDrvNum, TgtDrvType ) ;
   }

   return( retval ) ;
}
