	PAGE	60,132
NAME	SCAT
;	[ - condensed	Toshiba P351
	TITLE	SCAT Version 2.2
COMMENT |

SCAT.COM Copyright Roedy Green Canadian Mind Products (1988)
(1989) (1990) (1991)

May be freely distributed for non-military use only.

Program to fix up the BPB boot track on a floppy.  Works even if
the Media descriptor is damaged.  Primarily used to
decontaminate diskettes written by FastBack Plus.  Since
Fastback Plus overwrites the BPB boot track, such diskettes
masquerade as DOS diskettes. They fool some programs but not
others.  USE SCAT ON ALL SUCH DISKETTES BEFORE DOS REUSE.

Also fixes diskettes formatted under old versions of DOS.
Improves the "non system disk" message.

Runs under DOS versions 2.0 and up.

SCAT was based on Peter Fletcher's copyrighted FIX_BPB program

Please report bugs and problems to:
Roedy Green
Canadian Mind Products
#168 - 1020 Mainland Street
Vancouver BC Canada
V6B 2T4
(604) 684-6529
BIX id ROEDY
ENVOY id ROEDY
Zoomit id GREEN ROEDY

Version 2.2 February 27, 1991
-----------
- now recognizes Norton proprietary format diskettes

Version 2.1 July 28, 1990
-----------
- now upgrades diskettes to DOS 4.01 level.
- flashier banner.
- flashier accidental boot message
- "Love Roedy" removed from accidental reboot message.
- banner now describes the seven functions of SCAT in descending order
  of importance.
- box around major HIT prompt
- fix banner than mentioned 320 KB twice and left out 360 KB.
- new address and phone number

Version 2.0 July 27, 1990
-----------
- add 3.5" disk support
- add ability to SCAT either A: or B: drive
- now place 55AA signature at very end of boot block.

Version 1.3 August 10 1989
-----------

- Now put letters "IBM	3.3" in manufacturer ID field to more
  fully emulate IBM Diskettes.

Version 1.2 January 3 1989
-----------

Minor cosmetic changes to the wordings

Version 1.1  December 1988
-----------

- DOS 4.0/OS/2 reportedly started insisting the manufacture field
  contain the letters IBM.  We now humour them.

- We updated our address since we moved.

- The program is now copyrighted to give us more legal leverage
  to prevent military use.

How it works
------------

Here is our overall strategy to reconstruct the BPB.

1.  We read the boot record ourselves to determine the media
    descriptor byte. If it is undamaged, we can reconstruct the
    rest of the BPB.

2.  If the media byte is damaged, we might ask DOS its idea.
    The problem is, DOS will crash with divide overflow if it
    starts to interpret a mildly damaged BPB.  So we read the FAT
    instead and look for the media byte at the head of the FAT.

2.  If that does not work we ask DOS what the default BPB is for the device.

3.  Reconstruct the rest of the BPB from the media byte.

4.  Read track 0 to see what is there now.

5.  Warn user if we will actually be changing the BPB.

6.  Ask the user for final confirmation before fix.

However the simple algorithm above does not always work because
three types of floppy all use the same 0f9 media byte and two use
the same 0f0 media byte.  When that comes up, we need to discriminate
between them by comparing other fields.

This code was written "quick and dirty"  Do not use it as an
example of how to code.

Futures
-------

- handle 3.5 inch floppies and hard disks as well.

- during a floppy reboot, add code to wait till the user lets go
  of the key before rebooting to avoid spurious bad keyboard errors.

|	; end of comment

CR	MACRO	; Carriage return line feed
	DB 0dh,0ah
	ENDM

EOS	MACRO	; marks end of display string
	DB 0dh,0ah,'$'
	ENDM
;======


BPBRaw Struc		; The BPB is the part of the boot record
			; that describes the disk geometry
			; 21 bytes long
sect_siz	dw 200h ; All IBM compatible floppies have 512-byte sectors
clst_siz	db ?	; Number of sectors per cluster
res_sect	dw 1	; Number of reserved sectors on the disk
n_fats		db 2	; Number of FATs
n_entries	dw ?	; Number of entries in the Root Directory
n_sectors	dw ?	; Total number of sectors on the disk
descript	db ?	; Media descriptor byte
fat_size	dw ?	; Number of sectors in (one) FAT
t_sectors	dw ?	; Number of sectors per track
n_heads 	dw ?	; Number of heads (== sides)
n_hidden	dd 0	; Number of "hidden" sectors
BPBRaw		Ends

;======

BPBTail Struc		; 10 bytes
Reserved_1	dd ?
Reserved_2	db 4 dup (0)
BPBTail  Ends

;======

BIOSDATA    segment AT 40h	; dummy segment in low RAM
		org	72h
BIOSWarmFlag	dw	?
BIOSDATA	ends

;======

ROMBIOS 	segment AT 0f000h	; dummy segment inside ROM BIOS
		org	0fff0h
BIOSReset	label	far		; reset code at FFFF:0000
					; better represented as F000:FFF0
ROMBIOS 	ends

;======

CSEG	segment
	assume CS:CSEG, DS:CSEG, ES:CSEG, SS:CSEG
					; Since this is a .COM program
	org 100h			; Likewise

entry:
	jmp	start	; Jump over the data


;==========================================================
;
; B O O T   S E C T O R
;
; This is where we build the image of the
; boot sector 0 to place on the floppy.
;==========================================================

Boot_Sector	db 512d dup (?) ; Work area to read/build boot sector

		ORG	Boot_Sector
Boot_jump	db 3 dup (?)	; Jump to bootstrap code
Boot_oem	db 8 dup (?)	; Name of OEM


		ORG	Boot_Sector+0Bh
Boot_BPS	BPBRaw <>	; 21 bytes

		ORG	Boot_Sector+26h
Boot_ESignature db ?		; 29h to mark extension
Boot_VolID	db 4 dup (?)	; 8 hex digits e.g. 1D35-1ECB unique
Boot_VolLab	db 11d dup (?)	; 11 character volume label
Boot_FAT_Type	db 8d dup (?)	; FAT12___

		ORG	Boot_Sector+3eh
Boot_Code	DB ?		; boot code for accidental boot.
				; padded with 0.

		ORG	Boot_Sector+0A1h
Norton_Kilroy	DB	22d dup (?)
				; where Norton places
				; "Norton Backup Diskette"

		ORG	Boot_Sector+510d
Boot_Signature	DW ?		; 55AA last two bytes

;=============================================
;
; B O O T   S E C T O R   M O D E L S
;
; Models for the boot sector
; See Ray Duncan's Advanced MS DOS page 180 for Boot sector layout.
;
;=============================================

Norton_Signature	DB	"Norton Backup Diskette"


My_Jump:		; patch for first 3 bytes of BPB
	DB	0EBh,03Ch,090h
			; JMP SHORT Boot_Code NOP
			; hand assembled jump to My_Boot_Code

;=============================================
My_Oem_DOS3:
	DB  'IBM  3.3'	; your company name (DOS 4.0 only likes IBM)
;	     12345678	; temporarily till we get volume label

My_Oem_DOS4:
	DB  'IBM  4.0'	; your company name (DOS 4.0 only likes IBM)
;	     12345678	; even for DOS 4.01, still say DOS 4.0

;========
;   Skeleton BPBs for various types of diskette
;   Use these both to recognize and patch BPBs

;		<,secs/clust,,,,entries,sectors,media,secs/fat,secs/track,heads,,>

D160	BPBRaw	<,1,,,64,320,0feh,1,8,1,>
D160$	DB	'XT 5" 160 KB ancient Single sided 8-sector'
	DB	' presumed.'
	EOS

D320	BPBRaw	<,2,,,112,640,0ffh,1,8,2,>
D320$	DB	'XT 5" 320 KB ancient Double sided 8-sector'
	DB	' presumed.'
	EOS

D180	BPBRaw	<,1,,,64,360,0fch,2,9,1,>
D180$	DB	'XT 5" 180 KB ancient Single sided 9-sector'
	DB	' presumed.'
	EOS

D360	BPBRaw	<,2,,,112,720,0fdh,2,9,2,>
D360$	DB	'XT 5" 360 KB Standard 9-sector yellow-dot'
	DB	' presumed.'
	EOS


D144	BPBRaw	<,1,,,224,2880,0f0h,9,18,2,>
D144$	DB	'XT 3" 1.44 MB High density 18-sector green-dot'
	DB	' presumed.'
	EOS

;  NOTE  Following two types all share same 0f9 media descriptor
;	We can tell the 1.2 MB and the 720 KB apart by examining
;	the drive characteristics.


D12	BPBRaw	<,1,,,224,2400,0f9h,7,15,2,>
D12$	DB	'AT 5" 1.2 MB Standard High density 15-sector red-dot'
	DB	' presumed.'
	EOS

D720	BPBRaw	<,2,,,112,1440,0f9h,3,9,2,>
D720$	DB	'DOS 3.3 XT 3" 720 KB Standard 9-sector blue-dot'
	DB	' presumed.'
	EOS


;=============================================
My_Boot_FAT_Type	DB	'FAT12   '
;				 XXXXXXXX
;=============================================

;   Code that gets executed if attempt to boot from non-boot floppy by mistake
;   We do not execute it now.  We just put in on the floppy boot track.

My_Boot_Code:
	CLI			; prevent interrupts
	XOR	AX,AX		; init SS:SP
	MOV	SS,AX
	MOV	SP,7c00h
	MOV	AX,CS
	MOV	DS,AX
	CALL	whereisstring	; self relocating code
Place:
	CLD
NextChar:
	LODSB			; get next char of message
	OR	AL,AL
	JZ	StringDone	; done when hit 0 at end
				; of string
	MOV	BX,7		; foreground colour
	MOV	AH,0Eh		; display one char
	INT	10h		; video i/o
				; handles 07 beep too
	JMP	NextChar
StringDone:
;	Wait for him to hit space bar or other key
	XOR	AH,AH
	INT	16h
;    Final farewell beep to let him know reboot took
	MOV	AL,7
	MOV	BX,7		; foreground colour
	MOV	AH,0Eh		; display one char
	INT	10h		; video i/o to do the beep
;=============================
; now reboot
;	set system reset flag at 0040:0072 to 01234 for warm or 0 for cold
	mov	ax,BIOSDATA
	mov	ES,ax
	assume	ES:BIOSDATA
	mov	BIOSWarmFlag,01234h	; warm boot
					; far jump to FFFF:0000 alias F000:FFF0
	jmp	BIOSReset		; to simulate a Ctrl-Alt-Del
	assume	ES:nothing

;=========================
WhereIsString:
	pop	SI			; get return address
	push	SI			; to figure out where we are
	ADD	SI,TheString-Place
	RET
TheString:
	DB	07h		; be
	CR
	DB	' ERROR ۲ Attempt to boot from a non-system diskette.'
	CR
	CR
	DB	'To reboot from your hard disk, open the door of'
	CR
	DB	'floppy drive A: and hit the space bar.'
	CR
	CR
	DB	'To reboot from a floppy disk, insert a system disk in'
	CR
	DB	'floppy drive A: and hit the space bar.'
	CR
	DB	0				; marks the end of displayable part
	CR
	DB	'Love Roedy'			; here on visible only
	CR					; to those who snoop with NU
	DB	' SCAT 2.2 ۲'
	CR
	DB	'Canadian Mind Products'
	CR
	DB	'#168 - 1020 Mainland'
	CR
	DB	'Vancouver BC Canada'
	CR
	DB	'V6B 2T4'
	CR
	DB	'(604) 684-6529'		; If you make this section too long,
	CR					; it will simply be chopped to
	DB	0				; fit.


EndMy_Boot:

;=======
Get_Dev 	label	byte	; control block for DOS's idea
				; of what BPB should be like.
				; get BPB that build would return
				; Only use these in an emergency
				; when both BPB and FAT are damaged
    SpecialFunctions	db	1
    DeviceTYpe		db	?
    DeviceAttribs	dw	?
    NoOfCyls		dw	?
    MediaType		db	?
    DeviceBPB		BPBRaw	<>
    DeviceTail		BPBTail <>

;=======
; Control blocks used to find out the volume label

xbuff	db	64d dup (0)	; receives result of volume label request
	org	xbuff+8 	; looks like extended FCB when done.
My_VolLab	db	11d dup (' ')
				; Volume Label we will apply

xfcb	db	0ffh		; extended FCB for seeking volume label
				; ff means extended FCB
	db	5 dup (0)	; reserved
	db	8		; attribute bit for volume label
FDrive	db	0		; drive code 1=A 2=B
	db	11d dup ('?')	; wildcard filename
	db	25d dup (0)	; rest of FCB not used
;=======

; Build a unique Volume serial ID

My_VolID	db	4d dup (0)	;

;=======
;
;   Display Strings

copyright	label	byte
 CR
 db ' Scat 2.2 ۲'
 CR
 CR
 DB '     - Copyright Roedy Green Canadian Mind Products (1988) (1989) (1990) (1991).'
 CR
 db '       May be freely distributed and used for any purpose except military.'
 CR
 DB '     - Upgrades diskettes to DOS 4.01 standards without losing data.'
 CR
 db '     - Repairs damaged BPB (boot track) of a 160 KB, 180 KB, 320 KB, 360 KB,'
 CR
 db '       720 KB, 1.2 MB or 1.44 MB floppy usually recovering the files.'
 CR
 db '     - Improves the error message when you accidentally boot from a floppy.'
 CR
 db '     - Humours PC DOS and OS/2 by putting IBM in as the OEM name.'
 CR
 db '     - Tells you what kind of floppy diskette you have.'
 CR
 db '     - Decontaminates old FastBack Plus diskettes of the "unvirus".'
 CR
 db '     - Removes any virus hiding in the boot block of a floppy.'
 CR
 db '     - Detects Norton Proprietary Backup format diskettes.'
 CR
 CR
 DB '     - DO NOT USE on system bootable DOS diskettes.'
 CR
 DB '     - DO NOT USE on FastBack Plus diskettes containing useful data.'
 CR
 EOS

Usage		label	byte
	db	'Error in command line.'
	CR
	db	'Try SCAT A: or SCAT B:'
	EOS

resort2 	label	Byte
	db	'Resorting to sniffing the FAT to recover.'
	EOS

resort3 	label	Byte
	db	'Resorting to close your eyes and pray recovery method.'
	EOS

bad_desc	label	byte
	db	'Damaged or unrecognized BPB media descriptor byte.'
	EOS

bad_Fat 	label	Byte
	db	7,'Oh dear, damaged or unrecognized FAT media descriptor byte.'
	CR
	DB	'The FAT is probably trashed.  Use Norton NU/DE to recover.'
	EOS

Damaged 	label	Byte
	db	'Parts of the BPB (boot block) or signature are damaged.'
	EOS

NoDamage	label	byte
	db	'The BPB (boot block) and signature appear to be correct.'
	EOS

Is_Norton	label	Byte
	db	'This disk contains data in proprietary Norton Backup format.'
	CR
	db	'It must be completely reformatted before it can be used under DOS.'
	EOS
Confirm 	Label	Byte
	CR
	db	'Ŀ'
	CR
	db	' Hit Enter or the space bar to fix/improve the BPB,  Hit Esc to abort. '
	CR
	db	''
	CR
	EOS

bad_write	label	Byte
	db	7,'Cannot write modified boot sector to disk.'
	EOS

Alls_well	label	Byte
	db	'Diskette now fixed.'
	EOS

Did_nothing	label	Byte
	db	7,'Diskette left as is.'
	EOS

;======================================
;    V A R I A B L E S
;======================================

DriveNo DB	0	; A:=0 B:=1 floppy drive
			; which floppy we are working on.

BestBPB DW	0	; pointer to best model BPB

;======================================
;   M A I N L I N E   R O U T I N E
;======================================

start:

	lea	dx, copyright		; Announce ourselves
	Call	Say
	Call	Parse			; parse the command line
	Call	Analyze 		; analyze the command line
	Call	ReadBoot		; read sector 0 for ourselves
					; and see if can determine
					; the media descriptor byte
	Call	Handle_Norton		; Check for Norton Proprietary
	mov	al,Boot_BPS.Descript
	call	Guess
	JNC	GotIt
	lea	dx,bad_desc		; Report bad media byte
	Call	Say
	lea	dx,resort2		; Resort to stage 2
	Call	Say
	Call	ReadFat 		; Read Fat, second copy of
					; media desc byte at head.
	mov	al,Boot_Sector
	Call	Guess
	JNC	GotIt
; Shoot! Both copies of media byte are damaged
	lea	dx,Bad_Fat		; report Fat Damage
	Call	Say
; Try just presuming default BPB for that drive type.
	lea	dx,resort3		; Resort to stage 3
	Call	Say
	mov	SpecialFunctions,0	; get default BPB for device
	call	AskDosBPB		; AL has media desc byte
	call	Guess			; guess what kind of disk it is.
	JNC	GotIt			;
	JMP	Abort			; No idea what kind, give up
GotIt:
	Call	DetectDamage		; warn user of damage we
					; are about to fix.
	Call	ConfirmIt		; Confirm ok to proceed
; clear whole record before patching.  We build it totally from scratch.
	Call	Clear_Boot_Sector

; We first construct a DOS 3.3 boot block.
; We need an undamaged boot block to be able to read the VL in the
; root directory.  By making it DOS 3.3 style, DOS will not
; sneak around us and give us the volume label from the boot block.
; We want the VL in the root directory.
; Once we have it, we can put the VL into the boot block along with
; the other DOS 4.01 fields.

	Call	Build_DOS3_Boot 	; Build new DOS3.0 boot block in RAM
	Call	WriteBoot		; write out the newly constructed boot
	Call	Read_VolLab		; get Volume label from directory
	Call	Calc_VolID		; Calculate a pseudorandom Volume ID
	Call	Build_DOS4_Boot 	; Build new DOS3.0 boot block in RAM
	Call	WriteBoot		; write out the newly constructed boot
; windup
	lea	dx, alls_well		; tell went ok
	Call	Say
	mov	ax, 4c00h		; ERRORLEVEL = 0
	int	21h			; normal bye
;============================================

abort:
;	abort with errorlevel=4

	Lea	dx, Did_nothing
	Call	Say
					; error exit
	mov	ax, 4c04h		; ERRORLEVEL = 4
	int	21h			; DIE

;============================================

Say		Proc	Near
;	on entry DX points to a string to display
	MOV	AH,9
	Int	21h
	ret
Say		EndP

;======================

ReadBoot	Proc	Near

;	reads current boot record
	mov	al, DriveNo		; set up for read A=0 B=1
	mov	cx, 1			; 1 sector long
	mov	dx, 0			; offset sector 0
	lea	bx, Boot_Sector
	int	25h			; and do it
	pop	ax			; remove junk pushed by INT 25
	jnc	goodread		; if no problem, exit cleanly
;	cannot read, fake it with 0s
	Call	Clear_Boot_Sector
GoodRead:
	RET
ReadBoot	EndP

;===================

WriteBoot	Proc	Near
;	writes boot record we have built in Boot_Sector
	mov	al, DriveNo		; set up for write A=0 B=1
	mov	cx, 1
	mov	dx, 0
	lea	bx, Boot_Sector
	int	26h			; and do it
	pop	ax			; remove junk pushed by INT 26
	jnc	goodwrite		; if no problem, exit cleanly
	lea	dx, bad_write		; else, report
	Call	Say
	JMP	Abort
GoodWrite:
	mov	ah,0dh
	int	21h			; reset disk subsystem
					; This will encourage DOS to look
					; freshly at the repaired BPB

	RET
WriteBoot	EndP

;===================

ReadFAT 	Proc	Near
;	reads current Fat record -- uses Boot_Sector buffer though
	mov	al, DriveNo		; set up for read A=0 B=1
	mov	cx, 1			; one sector long
	mov	dx, 1			; 2nd sector offset 1
	lea	bx, Boot_Sector
	int	25h			; and do it
	pop	ax			; remove junk pushed by int 25
	jnc	GoodFatRead		; if no problem, exit cleanly
;	cannot read, fake it with 0s
	Call	Clear_Boot_Sector
GoodFatRead:
	RET
ReadFat 	EndP

;===================

Clear_Boot_Sector	PROC	Near
;	clear entire boot sector to zeroes, preparatory to filling in fields
	mov	cx,512d/2
	mov	ax,0
	lea	di, Boot_Sector
	rep	stosw
	RET
Clear_Boot_Sector	ENDP

;===================

AskDosBPB	PROC	Near
	; on entry SpecialFunctions is 1 or 0
	; on exit AL has the media descriptor byte
	; We ask DOS what it thinks media descriptor byte should look like
					; BPB looks like
	mov	ax,0440dh		; generic IOCTL
	mov	BL,DriveNo		; A: is drive 1, B is 2
	inc	BL
	mov	CX,00860h		; get device parms
	lea	dx, Get_Dev		; DS:DX pointer to BPB
	int	21h

	jnc	goodask 		; carry set if error
	mov	al,0			; fake a 0 as media descriptor
					; DOS could not read drive
	ret
goodask:
	mov	al, DeviceBPB.descript	; get this disk's media descriptor
	RET
AskDosBPB	ENDP

;===========================================

Handle_Norton	Proc	Near
	lea	si,Norton_Kilroy	; area in boot where Norton leaves mark.
	lea	di,Norton_Signature
	mov	cx,22d			; length of signature
	REPE cmpsb
	jne	Not_Norton
	lea	dx,Is_Norton		; it is a Norton Backup disk
	call	Say
	jmp	Abort

Not_Norton:
	ret
Handle_Norton	ENDP

;===========================================

Guess		Proc	Near
;	On entry AL contains a media descriptor byte.
;	We see if any diskette type matches it.
;	If so, we set BestBPB to point to a matching BPB
;	that we guess is correct based on media byte.
;	This is not fool proof as media bytes are not unique.
;	If not, we set the carry flag to indicate failure
TryD160:
	cmp	al, D160.descript	; next comparison
	jne	TryD320 		; not equal, try the next
	lea	si, D160.sect_siz	; set up for BPB rewrite
	lea	dx, D160$
	jmp	go_on			; and go do it
TryD320:
	cmp	al, D320.descript	; next comparison
	jne	TryD180 		; etc., etc.
	lea	si, D320.sect_siz
	lea	dx, D320$
	jmp	go_on
TryD180:
	cmp	al, D180.descript
	jne	TryD360
	lea	si, D180.sect_siz
	lea	dx, D180$
	jmp	go_on
TryD360:
	cmp	al, D360.descript
	jne	TryD144
	lea	si, D360.sect_siz
	lea	dx, D360$
	jmp	go_on
TryD144:
	cmp	al, D144.descript
	jne	TryD720
	lea	si, D144.sect_siz
	lea	dx, D144$
	jmp	go_on
TryD720:
;	We have a problem,  Both 720 KB and 1.2 MB drives use
;	same media byte.  Discriminate by drive type
	push	ax		; save media byte
	call	GetDriveType
	; BL = drive type 01=360 02=1.2 03=720 04=1.44
	pop	ax		; restore media byte
	cmp	bl,02d		; 1.2 MB drive is type 02
	je	ProbablyD12
	cmp	bl,03d		; 720 KB drive is type 03
	je	ProbablyD720
	cmp	bl,04d		; 1.44 MB drive is type 04
	je	ProbablyD720
	jmp	no_more 	; else something is amiss
ProbablyD12:
	cmp	al, D12.descript
	jne	no_more
	lea	si, D12.sect_siz
	lea	dx, D12$
	jmp	go_on
ProbablyD720:
	cmp	al, D720.descript
		jne	no_more
	lea	si, D720.sect_siz
	lea	dx, D720$
	jmp	go_on
no_more:				; couldn't find a match
	mov	BestBPB,SI
	stc
	ret
go_on:					; did find a match
	mov	BestBPB,SI
	Call	Say			; tell him what we presumed
	clc
	ret
Guess		EndP
;=============================================

GetDriveType	PROC	Near
	; on entry:
	; DriveNo is A:=0 B:=1
	; on exit:
	; BL = drive type 01=360 02=1.2 03=720 04=1.44
	mov	ah,0
	int	13h		; reset the floppy system
	mov	dh,0
	mov	dl,DriveNo
	push	ES
	mov	ah,8h		; code for current drive parms
	int	13h		; ( abs diskette i/o)
	pop	ES		; restore trashed ES register
	ret
GetDriveType	ENDP

;============================================

DetectDamage	Proc	Near
;	Detect if disk is currently damaged by comparing how
;	it is now with how it will be after it is repaired.
;	We only compare a crucial section, not the whole boot sector.
;	on entry BestBPB contains a pointer to the new proper BPB

	Call	ReadBoot		; re-read existing Boot 0 sector
					; We may have fiddled with current
					; in-RAM copy patching it with
					; the media byte etc.
	cmp	Boot_Signature,0AA55h	; signature ok?
	jne	Is_Damaged
;	see if we will actually be changing the BPB
	mov	cx, (size BPBRaw)	; number of bytes to compare
	mov	si, BestBPB		; pointer to replacement BPB
	lea	di, Boot_BPS.sect_siz	; destination
	repe	cmpsb			; compare current with proper BPB
	JE	Is_Not_Damaged
Is_Damaged:
	Lea	dx, Damaged		; It will be different,
	Call	Say			; warn user of current damage.
	RET
Is_Not_Damaged:
	Lea	dx, NoDamage		; tell user all is ok.
	Call	Say
	RET
DetectDamage	ENDP

;=============================================

ConfirmIt	Proc	Near
;	Confirm that ok to proceed with update of boot block
;	If, not jump directly to ABORT.

	Lea	DX,Confirm
	Call	Say
	mov	AH,07			; read one char, no echo
	int	21h
	cmp	AL,32			; is it a blank?
	JE	Confirmed
	cmp	AL,0dh			; is it Cr?
	JE	Confirmed
	JMP	Abort			; No, Bail out
Confirmed:				; Yes, carry on
	RET
ConfirmIt	EndP

;========================================

Build_DOS3_Boot PROC	Near

;	Build improved boot sector DOS 3.3 style

; patch short jump to actual boot code : EB 3C 90
	mov	cx,3
	lea	di, Boot_jump
	lea	si, My_jump
	rep	movsb
; patch in replacement BPB
	mov	cx, (size BPBRaw)	; number of bytes to move
					; sect-size thru nhidden
	mov	si,BestBPB		; DS:SI points to
					; replacement BPB
	lea	di, Boot_BPS.sect_siz	; destination
	rep	movsb			; move proper BPB to record
; patch oem
	mov	cx,8
	lea	di, Boot_oem
	lea	si, My_OEM_DOS3
	rep	movsb

; patch the boot code itself
	mov	cx,EndMy_Boot - My_Boot_Code
	lea	di, Boot_Code
	lea	si, My_Boot_Code
	rep	movsb

; patch the signature word
	mov	Boot_Signature,0AA55h
	ret

Build_DOS3_Boot ENDP

;========================================

Build_DOS4_Boot PROC	Near

;	Build improved boot track DOS 4.01 style

; patch short jump done already
; patch in replacement BPB done already
; patch oem
	mov	cx,8
	lea	di, Boot_oem
	lea	si, My_OEM_DOS4
	rep	movsb
; patch extended boot signature
	mov	Boot_ESignature,029H

; patch binary volume ID serial number	; Boot_VolID
					; 8 hex digits e.g. 1D35-1ECB unique
	mov	cx,4
	lea	di, Boot_VolID
	lea	si, My_VolID
	rep	movsb

; patch Boot_VolLab			; 11 char volume label name
	mov	cx,11d
	lea	di, Boot_VolLab
	lea	si, My_VolLab
	rep	movsb

; patch FAT_Type			; patch in FAT12___
	mov	cx,8
	lea	di,Boot_FAT_Type
	lea	si,My_Boot_FAT_Type
	rep	movsb

; patch the boot code already done

; patch the signature word already done
	ret

Build_DOS4_Boot ENDP

;========================================

Read_VolLab	PROC	Near
;	We search for the 11-character Volume Label in the root directory
;	We ignore the one that was might have been in the boot sector.
;	We leave the result in My_VolLab.
;	We use FCBs instead of handles so that SCAT will work under DOS 2.0.

	lea	dx,xbuff
	mov	ah,1ah
	int	21h		; set dta
	mov	al,DriveNo	; 0=A: 1=B:
	inc	al
	mov	FDrive,al	; 1=A: 2=B: in FCB
	lea	dx,xfcb
	mov	ah,11h
	int	21h		; search for first file match using XFCB
				; al=ff means no match
				; We do not bother to test.
				; if none, VolLab is still blank from init.
				; if name is short, DOS pads with blanks
	RET
Read_VolLab	ENDP

;========================================

Calc_VolID	PROC	Near
;	VolID is a 32-bit binary number.  Ideally it should be unique
;	in all the universe.  We generate a pseudo-random number this way:
;	lsw  = (hundredths of sec) + Day + [(sec + month) * 256]
;	msw  =	Year + Min + (hour *256)
;	This is the way format does it.
	mov	ah,2ch
	int	21h			; get dos time
	add	My_Volid+3,ch		; ch = hh
	add	My_Volid+2,cl		; cl = min
	add	My_Volid+1,dh		; dh = sec
	add	My_Volid+0,dl		; dl = hundredths
	mov	ah,2ah
	int	21h			; get dos date
	add	word ptr My_Volid+2,cx	; cx = yyyy
	add	My_Volid+1,dh		; dh = mm
	add	My_Volid+0,dl		; dl = dd
	RET
Calc_VolID	ENDP

;========================================

MLeading	PROC	Near
;	Remove leading blanks
;	on entry BX is addr of string, CX its length
;	trims off any leading blanks, leaving result in BX CX
;	length may also be 0 or 1, but not -ve
;	If the entire string is blank the result is the null string
	mov	di,bx
	mov	al,20H		; AL = blank  -- the search char
	jcxz	mleading2	; jump if null string
	repe	scasb		; scan ES:DI forwards till hit non blank
				; DI points just after it (wrap ok)
				; cx IS ONE TOO SMALL, OR 0 IF NONE FOUND
	je	mleading1	; jump if entire string was blank
	inc	cx		; CX is length of remainder of string
mleading1:
	dec	di		; DI points to non-blank
mleading2:
	mov	bx,di		; put address back
	ret
MLeading	ENDP

;========================================

MTrailing	PROC	Near
;	Remove trailing blanks.
;	on entry BX is addr of string, CX its length
;	trims off any trailing blanks, leaving result in BX CX
;	length may also be 0 or 1, but not -ve
;	If the entire string is blank the result is the null string
	mov	di,bx
	add	di,cx		; calc addr last char in string
	dec	di
	mov	al,20H		; AL = blank  -- the search char
	jcxz	mtrailing1	; jump if null string
	std
	repe	scasb		; scan ES:DI backwards till hit non blank
				; DI points just ahead of it (wrap ok)
				; CX is one too small, or 0 if none found
	cld
	je	mtrailing1	; jump if whole string was blank
	inc	cx
mtrailing1:
	ret
MTrailing	ENDP

;========================================

Parse		PROC	NEAR
;	Parse the command line to remove lead/trail blanks from
;	the single drive parameter and terminate it by 2 nulls.
;	sample inputs
;	SCAT A:
;	SCAT   B:
;	SCAT
;
;	When Done DS:BX points to start of string.
;	String will be terminated by 2 nulls
;	CX counts bytes in string exclusive of nulls
				; counted string at HEX 80 PSP
				; contains command line.
				; Preceeded by unwanted spaces.
				; possibly followed by unwanted spaces.
				; currently missing a trailing null.
	xor	ch,ch
	mov	cl,ds:80H
	mov	bx,81H
	call	Mleading	; get rid of leading blanks
	call	MTrailing	; get rid of trailing blanks
	mov	di,bx		; calc addr of byte just past end
	add	di,cx
	mov	word ptr [di],0 ; plop in pair of nulls after string
	ret
Parse		ENDP

;======================================

ToUC		PROC	NEAR
;	converts char in AL to upper case
	cmp	al,'a'
	jb	FineAsIs
	cmp	al,'z'
	ja	FineAsIs
	sub	al,20H		; convert a to A
FineAsIs:
	ret
ToUc		ENDP

;======================================

Analyze 	PROC	NEAR
;	analyses the command line allowing A: or B: or null
;	On entry DS:BX points to start of string.
;	String will be terminated by 2 nulls
;	CX counts bytes in string exclusive of nulls
;	lead/trail spaces are gone.
	mov	DriveNo,0
	jcxz	AnalDone	; was no command, treat as A:
	cmp	cx,2
	jne	BadCmd		; kick out A   A:/X
	mov	ax,[bx] 	; get chars AL<-A  AH<-:
	cmp	ah,':'		; make sure second char is :
	jne	BadCmd
	call	ToUc
	sub	al,'A'		; a->0 b->1
	cmp	al,1
	ja	BadCmd		; allow only A: and B:
	mov	DriveNo,AL
AnalDone:
	RET
BadCmd:
	LEA	dx,Usage
	Call	Say
	Jmp	Abort
Analyze 	ENDP

;======================================

CSEG ends
end entry
