; slipflop.asm
;
; Modifies boot sector of diskette, which becomes a Slippy Floppy.
; I can't see any real use, except perhaps one circumstance:
; debugging program, on machine which has a single floppy drive, which is
; used for intermediary backup of work in progress. Numerous crashes,
; multiple reboots, may avoid tediousness of forgetting open/closing floppy
; drive's door.
; Relatively compact. It lets BIOS handle problem of different disks formats,
; preserves previously laid DOS disk info. Replacement boot sector is embedded
; and assembled at assembly time, doesn't require separate data file.
; (Code is part of work in progress for alternatives to boot procedures
; of non-DOS operating systems).
;
; Assembly: tasm, tlink, exe2bin slipflop, del sliflop.exe
; Usage: slipflop
; no argument, defaults drive A:
;
; Vincent Vinh-Hung
; 27-Dec-91
;

; issue start msg
; read old boot sector to secbuf
; move old bootinfo to new boot
; move new boot to secbuf
; write secbuf
; end

extrn   nowhere:far

bootsector      struc
        bsJump          db 3 dup (?)
        bsOemName       db 'Oem_Name'
        bsBytesPerSec   dw ?
        bsSecPerClust   db ?
        bsResSectors    dw ?
        bsFATs          db ?
        bsRootDirEnts   dw ?
        bsSectors       dw ?
        bsMedia         db ?
        bsFATsecs       dw ?
        bsSecPerTrack   dw ?
        bsHeads         dw ?
        bsHiddenSecs    dd ?
        bsHugeSectors   dd ?
        bsDriveNumber   db ?
        bsReserved1     db ?
        bsBootSignature db ?
        bsVolumeID      dd ?
        bsVolumeLabel   db 'VolumeLabel'
        bsFileSysType   db 'FileSyst'
bootsector      ends

bsvl    = 11
bsSize  = SIZE bootsector
floppy0 = 0             ; floppy drive A
hd0     = 80h           ; fixed disk 0
biosBLA = 7c00h         ; BIOS boot load address
bBLAshr = 7c0h          ; BIOS boot load address SHR 4
retryct = 3
maxsecsz= 2000h
bsSign  = 0aa55h
bsSignAd= 1feh

cseg    segment para public 'CODE'

        org     0
addr0   equ     $

        org     100h

        assume  cs:cseg, ds:cseg, es:cseg, ss:cseg

machrec        proc near

start:
        mov     word ptr [sav_es], es
        mov     ax, es                  
        mov     bx, offset secbuf       ; converts seg:off to seg:0
        mov     cl, 4
        shr     bx,cl
        add     ax, bx
        mov     es, ax
        cli
        mov     word ptr [savsp], sp
        mov     sp, offset topstack
        sti
        mov     word ptr [warmb5 - 4], biosBLA
        mov     word ptr [warmb5 - 2], 0

; insert blank newly formatted diskette

        mov     si, offset msg_insert_disk
        call    telew0
        mov     ah, 0
        int     16h

; read diskette boot sector

        mov     ah,0
        mov     dl, floppy0
        int     13h
        mov     cx, retryct
        xor     bx, bx                  ; es:0 is secbuf
        mov     dh, 00
        mov     dl, floppy0
rdbs1:  push    cx
        mov     cx, 1
        mov     ax, 0201h
        int     13h
        pop     cx
        jnc     rdbs2
        loop    rdbs1
        jmp     short pgmbug

; get old bootinfo

rdbs2:
        push    es
        mov     ax, ds
        mov     es, ax
        pop     ds
        mov     si, (SIZE bsJump)
        mov     di, offset bootinfo + (SIZE bsJump)
        mov     cx, (bsSize - SIZE bsJump)
        rep     movsb
        push    ds
        mov     ax, es
        mov     ds, ax
        pop     es

; move new boot to secbuf

        mov     si, offset bootinfo
        mov     di, 0
        mov     cx, (offset endNewBoot - offset bootinfo)
        rep     movsb

; write new boot sector

        xor     bx, bx
        mov     dh, 0
        mov     dl, floppy0
        mov     cx, retryct
wrbs1:  push    cx
        mov     cx, 1
        mov     ax, 0301h
        int     13h
        pop     cx
        jnc     wrbs2
        loop    wrbs1
        jmp     short pgmbug
wrbs2:
        mov     si, offset msg_slipf_done
        call    telew0
        mov     al, 0
        jmp     short pgmterm

pgmbug: mov     si, offset msg_write_err
        call    telew0
        mov     al, 1
pgmterm:
        mov     es, word ptr [sav_es]
        mov     sp, word ptr [savsp]
        mov     ah, 4ch
        int     21h

; new boot sector: begin

bootinfo:
nbs1    equ $
        jmp     short nbs2
        nop
        org     nbs1
        bootsector      <>
        db      '*** Unbootable Slippy Floppy ***'
nbs2:
        mov     ax, bBLAshr
        cli
        mov     ss, ax
        mov     ds, ax
        xor     ax, ax
        mov     sp, ax
        sti
        push    bBLAshr                         ; 0:7c00 not same 07c0:0 !
        push    (offset nbs3 - offset nbs1)     ; converts cs:ip to 07c0:xxxx
        retf
nbs3:
        mov     si, (offset msg_slippy - offset nbs1)
        call    telew0

warmb0:                                 ; relocate code at cs:maxsecsz
        mov     di, maxsecsz
        mov     si, (offset warmb1 - offset nbs1)
        mov     ax, bBLAshr
        mov     es, ax
        mov     cx, (offset warmb5 - offset warmb1)
        rep     movsb
        mov     ax, maxsecsz
        jmp     ax
warmb1:
        mov     cx, retryct
warmb2: push    cx
        mov     ax, 201h
        mov     cx, 1
        mov     dx, hd0
        xor     bx, bx
        int     13h
        pop     cx
        jnc     warmb3
        loop    warmb2
warmb4: int     18h
warmb3: mov     bx, bsSignAd
        mov     ax, word ptr [bx]
        cmp     ax, bsSign
        jne     warmb4
        jmp     far ptr nowhere         ; jmp to 0000:7c00h
warmb5  equ     $

; new boot sector: subroutine write teletype null-terminated string

telew1: mov     ah, 0eh
        xor     bx, bx
        int     10h
telew0: lodsb
        or      al, al
        jnz     telew1
        ret

msg_slippy     db      'Skip floppy', 0dh, 0ah, 0

endNewBoot      equ     $

sav_es          dw      ?
savsp           dw      ?
msg_insert_disk db      'Insert formatted diskette', 0dh, 0ah
                db      'press a key when ready', 0dh, 0ah, 0
msg_slipf_done  db      'Slippy Floppy, done.', 0dh, 0ah, 0
msg_write_err   db      'Write error, abort.', 0dh, 0ah, 0

; pad to align on paragraph
temp2 = 16*(($-addr0+15)/16) - ($-addr0)
if temp2 NE 0
                db      temp2 dup (0ffh)
endif
; stack space
                db      128 dup (55h)
topstack        equ     $
secbuf          db      maxsecsz dup (?)

machrec        endp

cseg    ends

        end     machrec

